#cs ----------------------------------------------------------------------------

 Copyright (C) 2015 Sebastian Walter (chesstiger)

 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.

 AutoIt Version: 			3.3.12.0
 InputFilter.au3 Version: 	1.0
#ce ----------------------------------------------------------------------------
#include <Memory.au3>
#include <SendMessage.au3>
#include <WinAPI.au3>
#include <WindowsConstants.au3>
#include <EditConstants.au3>
#include-once

;const
Global Enum $INPUTFILTER_ALLOW, $INPUTFILTER_DENY, $INPUTFILTER_SILENTDENY

;init
Global $__h_InputFilter_WndProc = DllCallbackRegister(__InputFilter_WndProc, "long", "hwnd;uint;uint;long")
Global $__p_InputFilter_WndProc = DllCallbackGetPtr($__h_InputFilter_WndProc)
OnAutoItExitRegister(__InputFilter_Exit)

; #FUNCTION# ====================================================================================================================
; Name ..........: GUICtrlInputSetFilter
; Description ...: Sets an input filter function for an input control.
;                  This filter func is called with two arguments, the pressed key
;                  and the resulting text. The return value of this function
;                  decides if the pressed char is allowed or denied,
;                  see the Enum on top of the file.
; Syntax ........: GUICtrlInputSetFilter($cInput, $sFilterFunc)
; Parameters ....: $cInput              - control ID
;                  $sFilterFunc         - filter func
; Return values .: None
; Author ........: Sebastian Walter (chesstiger)
; Modified ......:
; Remarks .......:
; Related .......:
; Link ..........:
; Example .......: Yes (InputFilter Example.au3)
; ===============================================================================================================================
Func GUICtrlInputSetFilter($cInput, $sFilterFunc)
	$sFilterFunc = String($sFilterFunc)
	$hInput = GUICtrlGetHandle($cInput)
	;prepare info struct
	$pInfoStruct = _MemGlobalAlloc(8 + 4 + 256)
	$tInfoStruct = DllStructCreate("byte[8];long_ptr oldWndProc;char filterFunc[256]", $pInfoStruct)
	$tInfoStruct.oldWndProc = _WinAPI_GetWindowLong($hInput, $GWL_WNDPROC)
	$tInfoStruct.filterFunc = $sFilterFunc
	;store *struct as userdata
	_WinAPI_SetWindowLong($hInput, $GWL_USERDATA, DllStructGetPtr($tInfoStruct))
	;modify wndproc
	_WinAPI_SetWindowLong($hInput, $GWL_WNDPROC, $__p_InputFilter_WndProc)
EndFunc   ;==>GUICtrlInputSetFilter

;LRESULT CALLBACK WndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
Func __InputFilter_WndProc($hWnd, $iMsg, $iWParam, $iLParam)
	$pInfoStruct = _WinAPI_GetWindowLong($hWnd, $GWL_USERDATA)
	$tInfoStruct = DllStructCreate("byte[8];long_ptr oldWndProc;char filterFunc[256]", $pInfoStruct)

	If $iMsg = $WM_CHAR Then
		Switch $iWParam
			Case 0 To 31
			Case Else
				;we have to build the result string for further processing
				;this requires the current selection by EM_GETSEL
				$iSelInfo = _SendMessage($hWnd, $EM_GETSEL, 0, 0, 0)
				$iFirstChar = _WinAPI_LoWord($iSelInfo)
				$iLastChar = _WinAPI_HiWord($iSelInfo)
				$sCurString = _WinAPI_GetWindowText($hWnd)
				$sModString = StringLeft($sCurString, $iFirstChar) & Chr($iWParam) & StringRight($sCurString, StringLen($sCurString) - $iLastChar)
				;call filterfunc
				Switch Call(String($tInfoStruct.filterFunc), Chr($iWParam), $sModString)
					Case $INPUTFILTER_ALLOW
						;nop
					Case $INPUTFILTER_DENY
						__InputFilter_ShowBalloonError($hWnd, "Unzulssige Eingabe", "Das Zeichen '" & Chr($iWParam) & "' ist an dieser Stelle unzulssig.")
						ContinueCase
					Case $INPUTFILTER_SILENTDENY
						Return 0
				EndSwitch
		EndSwitch
	EndIf

	If $iMsg = $WM_DESTROY Then ;clean up!
		$pOrgWndProc = $tInfoStruct.oldWndProc
		_MemGlobalFree($pInfoStruct)
		_WinAPI_SetWindowLong($hWnd, $GWL_WNDPROC, $pOrgWndProc)
		Return _WinAPI_CallWindowProc($pOrgWndProc, $hWnd, $iMsg, $iWParam, $iLParam)
	EndIf

	Return _WinAPI_CallWindowProc($tInfoStruct.oldWndProc, $hWnd, $iMsg, $iWParam, $iLParam)
EndFunc   ;==>__InputFilter_WndProc

Func __InputFilter_Exit()
	DllCallbackFree($__h_InputFilter_WndProc)
EndFunc   ;==>__InputFilter_Exit

Func __InputFilter_ShowBalloonError($hWnd, $sTitle, $sText)
	$tagEDITBALLOONTIP = "dword cbStruct;ptr pszTitle;ptr pszText;int ttiIcon"

	$tEBT = DllStructCreate($tagEDITBALLOONTIP)
	$tEBT.cbStruct = DllStructGetSize($tEBT)
	$tData = DllStructCreate(StringFormat("wchar title[%d];wchar text[%d]", StringLen($sTitle) + 1, StringLen($sText) + 1))
	$tData.title = $sTitle
	$tData.text = $sText
	$tEBT.pszTitle = DllStructGetPtr($tData, "title") & Chr(0)
	$tEBT.pszText = DllStructGetPtr($tData, "text") & Chr(0)
	$tEBT.ttiIcon = 3

	_SendMessage($hWnd, $EM_SHOWBALLOONTIP, 0, DllStructGetPtr($tEBT))
EndFunc   ;==>__InputFilter_ShowBalloonError
